@@ -133,6 +133,8 @@ main (int argc, char **argv)
     gboolean no_keep_image_data = FALSE;
     GError *error = NULL;
 
+    gboolean success = TRUE;
+
     int i;
     char **args = NULL;
     gint n_args = 0;
@@ -262,7 +264,7 @@ main (int argc, char **argv)
     if (keep_image_data)
         flags |= RSVG_HANDLE_FLAG_KEEP_IMAGE_DATA;
 
-    for (i = 0; i < n_args; i++) {
+    for (i = 0; i < n_args && success; i++) {
         GFile *file;
         GInputStream *stream;
 
@@ -316,8 +318,10 @@ main (int argc, char **argv)
         if (i == 0) {
             struct RsvgSizeCallbackData size_data;
 
-            if (!rsvg_handle_get_dimensions_sub (rsvg, &dimensions, export_lookup_id))
+            if (!rsvg_handle_get_dimensions_sub (rsvg, &dimensions, export_lookup_id)) {
                 g_printerr ("Could not get dimensions for file %s\n", args[i]);
+                exit (1);
+            }
 
             unscaled_width = dimensions.width;
             unscaled_height = dimensions.height;
@@ -465,7 +469,10 @@ main (int argc, char **argv)
         cairo_scale (cr,
                      scaled_width / unscaled_width,
                      scaled_height / unscaled_height);
-        rsvg_handle_render_cairo_sub (rsvg, cr, export_lookup_id);
+        if (!rsvg_handle_render_cairo_sub (rsvg, cr, export_lookup_id)) {
+            g_printerr ("Could not render file %s\n", args[i]);
+            exit (1);
+        }
 
         g_free (export_lookup_id);
 
