@@ -160,8 +160,8 @@ impl AspectRatio {
                 .and_then(|ident| Align::parse_xy(ident))
         })?;
 
-        let fit =
-            p.try(|p| {
+        let fit = p
+            .try(|p| {
                 p.expect_ident()
                     .map_err(|_| ())
                     .and_then(|ident| FitMode::parse(ident))
