@@ -18,7 +18,7 @@ use surface_utils::{
 };
 use util::clamp;
 
-use super::context::{FilterContext, FilterOutput, FilterResult};
+use super::context::{FilterContext, FilterOutput, FilterResult, IRect};
 use super::input::Input;
 use super::{Filter, FilterError, PrimitiveWithInput};
 
@@ -95,6 +95,55 @@ impl NodeTrait for Composite {
     }
 }
 
+/// Performs the arithmetic composite operation. Public for benchmarking.
+#[inline]
+pub fn composite_arithmetic(
+    input_surface: &SharedImageSurface,
+    input_2_surface: &SharedImageSurface,
+    output_surface: &mut cairo::ImageSurface,
+    bounds: IRect,
+    k1: f64,
+    k2: f64,
+    k3: f64,
+    k4: f64,
+) {
+    let output_stride = output_surface.get_stride() as usize;
+    {
+        let mut output_data = output_surface.get_data().unwrap();
+
+        for (x, y, pixel, pixel_2) in Pixels::new(input_surface, bounds)
+            .map(|(x, y, p)| (x, y, p, input_2_surface.get_pixel(x, y)))
+        {
+            let i1a = f64::from(pixel.a) / 255f64;
+            let i2a = f64::from(pixel_2.a) / 255f64;
+            let oa = k1 * i1a * i2a + k2 * i1a + k3 * i2a + k4;
+            let oa = clamp(oa, 0f64, 1f64);
+
+            // Contents of image surfaces are transparent by default, so if the resulting pixel is
+            // transparent there's no need to do anything.
+            if oa > 0f64 {
+                let compute = |i1, i2| {
+                    let i1 = f64::from(i1) / 255f64;
+                    let i2 = f64::from(i2) / 255f64;
+
+                    let o = k1 * i1 * i2 + k2 * i1 + k3 * i2 + k4;
+                    let o = clamp(o, 0f64, oa);
+
+                    ((o * 255f64) + 0.5) as u8
+                };
+
+                let output_pixel = Pixel {
+                    r: compute(pixel.r, pixel_2.r),
+                    g: compute(pixel.g, pixel_2.g),
+                    b: compute(pixel.b, pixel_2.b),
+                    a: ((oa * 255f64) + 0.5) as u8,
+                };
+                output_data.set_pixel(output_stride, output_pixel, x, y);
+            }
+        }
+    }
+}
+
 impl Filter for Composite {
     fn render(
         &self,
@@ -135,47 +184,21 @@ impl Filter for Composite {
                 input.surface().height(),
             )?;
 
-            let output_stride = output_surface.get_stride() as usize;
-            {
-                let mut output_data = output_surface.get_data().unwrap();
+            let k1 = self.k1.get();
+            let k2 = self.k2.get();
+            let k3 = self.k3.get();
+            let k4 = self.k4.get();
 
-                let k1 = self.k1.get();
-                let k2 = self.k2.get();
-                let k3 = self.k3.get();
-                let k4 = self.k4.get();
-
-                for (x, y, pixel, pixel_2) in Pixels::new(input.surface(), bounds)
-                    .map(|(x, y, p)| (x, y, p, input_2.surface().get_pixel(x, y)))
-                {
-                    let i1a = f64::from(pixel.a) / 255f64;
-                    let i2a = f64::from(pixel_2.a) / 255f64;
-                    let oa = k1 * i1a * i2a + k2 * i1a + k3 * i2a + k4;
-                    let oa = clamp(oa, 0f64, 1f64);
-
-                    // Contents of image surfaces are transparent by default, so if the
-                    // resulting pixel is transparent there's no need
-                    // to do anything.
-                    if oa > 0f64 {
-                        let compute = |i1, i2| {
-                            let i1 = f64::from(i1) / 255f64;
-                            let i2 = f64::from(i2) / 255f64;
-
-                            let o = k1 * i1 * i2 + k2 * i1 + k3 * i2 + k4;
-                            let o = clamp(o, 0f64, oa);
-
-                            (o * 255f64).round() as u8
-                        };
-
-                        let output_pixel = Pixel {
-                            r: compute(pixel.r, pixel_2.r),
-                            g: compute(pixel.g, pixel_2.g),
-                            b: compute(pixel.b, pixel_2.b),
-                            a: (oa * 255f64).round() as u8,
-                        };
-                        output_data.set_pixel(output_stride, output_pixel, x, y);
-                    }
-                }
-            }
+            composite_arithmetic(
+                input.surface(),
+                input_2.surface(),
+                &mut output_surface,
+                bounds,
+                k1,
+                k2,
+                k3,
+                k4,
+            );
 
             output_surface
         } else {
