@@ -40,7 +40,7 @@ pub use drawing_ctx::{
     rsvg_drawing_ctx_new,
 };
 
-pub use load::{rsvg_load_new_node, rsvg_load_set_node_atts};
+pub use load::{rsvg_load_new_node, rsvg_load_set_node_atts, rsvg_load_set_svg_node_atts};
 
 pub use node::{
     rsvg_node_add_child,
@@ -49,11 +49,16 @@ pub use node::{
     rsvg_node_children_iter_next,
     rsvg_node_find_last_chars_child,
     rsvg_node_get_parent,
-    rsvg_node_get_type,
-    rsvg_node_is_same,
     rsvg_node_ref,
     rsvg_node_unref,
-    rsvg_root_node_cascade,
+};
+
+pub use tree::{
+    rsvg_tree_cascade,
+    rsvg_tree_free,
+    rsvg_tree_get_root,
+    rsvg_tree_is_root,
+    rsvg_tree_new,
 };
 
 pub use parsers::rsvg_css_parse_number_optional_number;
@@ -68,10 +73,13 @@ pub use property_bag::{
 
 pub use state::{rsvg_state_free, rsvg_state_new, rsvg_state_parse_style_pair};
 
-pub use structure::{rsvg_node_svg_apply_atts, rsvg_node_svg_get_size};
+pub use structure::rsvg_node_svg_get_size;
 
 pub use text::{rsvg_node_chars_append, rsvg_node_chars_new};
 
+#[macro_use]
+mod log;
+
 #[macro_use]
 mod coord_units;
 
@@ -91,6 +99,7 @@ mod defs;
 mod drawing_ctx;
 mod error;
 pub mod filters;
+mod font_props;
 mod gradient;
 mod handle;
 mod image;
@@ -110,13 +119,14 @@ mod property_bag;
 mod rect;
 mod shapes;
 mod space;
-mod srgb;
+pub mod srgb;
 mod state;
 mod stop;
 mod structure;
 pub mod surface_utils;
 mod text;
 mod transform;
+mod tree;
 mod unitinterval;
 mod util;
 mod viewbox;
