@@ -126,7 +126,7 @@ impl NodeMarker {
         computed_angle: f64,
         line_width: f64,
         clipping: bool,
-    ) {
+    ) -> Result<(), RenderingError> {
         let cascaded = node.get_cascaded_values();
         let values = cascaded.get();
 
@@ -136,7 +136,7 @@ impl NodeMarker {
         if marker_width.approx_eq_cairo(&0.0) || marker_height.approx_eq_cairo(&0.0) {
             // markerWidth or markerHeight set to 0 disables rendering of the element
             // https://www.w3.org/TR/SVG/painting.html#MarkerWidthAttribute
-            return;
+            return Ok(());
         }
 
         let cr = draw_ctx.get_cairo_context();
@@ -189,13 +189,15 @@ impl NodeMarker {
             }
         }
 
-        draw_ctx.with_discrete_layer(node, values, clipping, &mut |dc| {
-            node.draw_children(&cascaded, dc, clipping);
+        let res = draw_ctx.with_discrete_layer(node, values, clipping, &mut |dc| {
+            node.draw_children(&cascaded, dc, clipping)
         });
 
         draw_ctx.pop_view_box();
 
         cr.restore();
+
+        res
     }
 }
 
@@ -381,15 +383,54 @@ pub fn path_builder_to_segments(builder: &PathBuilder) -> Vec<Segment> {
                 state = SegmentState::InSubpath;
             }
 
-            PathCommand::CurveTo((x2, y2), (x3, y3), (x4, y4)) => {
-                cur_x = x4;
-                cur_y = y4;
+            PathCommand::CurveTo(curve) => {
+                let CubicBezierCurve {
+                    pt1: (x2, y2),
+                    pt2: (x3, y3),
+                    to,
+                } = curve;
+                cur_x = to.0;
+                cur_y = to.1;
 
                 segments.push(make_curve(last_x, last_y, x2, y2, x3, y3, cur_x, cur_y));
 
                 state = SegmentState::InSubpath;
             }
 
+            PathCommand::Arc(arc) => {
+                cur_x = arc.to.0;
+                cur_y = arc.to.1;
+
+                match arc.center_parameterization() {
+                    ArcParameterization::CenterParameters {
+                        center,
+                        radii,
+                        theta1,
+                        delta_theta,
+                    } => {
+                        let rot = arc.x_axis_rotation;
+                        let theta2 = theta1 + delta_theta;
+                        let n_segs = (delta_theta / (PI * 0.5 + 0.001)).abs().ceil() as u32;
+                        let d_theta = delta_theta / f64::from(n_segs);
+
+                        let segment1 = arc_segment(center, radii, rot, theta1, theta1 + d_theta);
+                        let segment2 = arc_segment(center, radii, rot, theta2 - d_theta, theta2);
+
+                        let (x2, y2) = segment1.pt1;
+                        let (x3, y3) = segment2.pt2;
+                        segments.push(make_curve(last_x, last_y, x2, y2, x3, y3, cur_x, cur_y));
+
+                        state = SegmentState::InSubpath;
+                    }
+                    ArcParameterization::LineTo => {
+                        segments.push(make_line(last_x, last_y, cur_x, cur_y));
+
+                        state = SegmentState::InSubpath;
+                    }
+                    ArcParameterization::Omit => {}
+                }
+            }
+
             PathCommand::ClosePath => {
                 cur_x = subpath_start_x;
                 cur_y = subpath_start_y;
@@ -588,7 +629,7 @@ fn emit_marker_by_name(
     computed_angle: f64,
     line_width: f64,
     clipping: bool,
-) {
+) -> Result<(), RenderingError> {
     if let Some(acquired) = draw_ctx.get_acquired_node_of_type(Some(name), NodeType::Marker) {
         let node = acquired.get();
 
@@ -602,7 +643,9 @@ fn emit_marker_by_name(
                 line_width,
                 clipping,
             )
-        });
+        })
+    } else {
+        Ok(())
     }
 }
 
@@ -618,8 +661,9 @@ fn emit_marker<E>(
     marker_type: MarkerType,
     orient: f64,
     emit_fn: &mut E,
-) where
-    E: FnMut(MarkerType, f64, f64, f64),
+) -> Result<(), RenderingError>
+where
+    E: FnMut(MarkerType, f64, f64, f64) -> Result<(), RenderingError>,
 {
     let (x, y) = match *segment {
         Segment::Degenerate { x, y } => (x, y),
@@ -630,7 +674,7 @@ fn emit_marker<E>(
         },
     };
 
-    emit_fn(marker_type, x, y, orient);
+    emit_fn(marker_type, x, y, orient)
 }
 
 pub fn render_markers_for_path_builder(
@@ -638,11 +682,11 @@ pub fn render_markers_for_path_builder(
     draw_ctx: &mut DrawingCtx,
     values: &ComputedValues,
     clipping: bool,
-) {
+) -> Result<(), RenderingError> {
     let line_width = values.stroke_width.0.normalize(values, draw_ctx);
 
     if line_width.approx_eq_cairo(&0.0) {
-        return;
+        return Ok(());
     }
 
     let marker_start = &values.marker_start.0;
@@ -650,7 +694,7 @@ pub fn render_markers_for_path_builder(
     let marker_end = &values.marker_end.0;
 
     match (marker_start, marker_mid, marker_end) {
-        (&IRI::None, &IRI::None, &IRI::None) => return,
+        (&IRI::None, &IRI::None, &IRI::None) => return Ok(()),
         _ => (),
     }
 
@@ -670,15 +714,20 @@ pub fn render_markers_for_path_builder(
                     computed_angle,
                     line_width,
                     clipping,
-                );
+                )
+            } else {
+                Ok(())
             }
         },
-    );
+    )
 }
 
-fn emit_markers_for_path_builder<E>(builder: &PathBuilder, emit_fn: &mut E)
+fn emit_markers_for_path_builder<E>(
+    builder: &PathBuilder,
+    emit_fn: &mut E,
+) -> Result<(), RenderingError>
 where
-    E: FnMut(MarkerType, f64, f64, f64),
+    E: FnMut(MarkerType, f64, f64, f64) -> Result<(), RenderingError>,
 {
     enum SubpathState {
         NoSubpath,
@@ -705,7 +754,7 @@ where
                         MarkerType::End,
                         angle_from_vector(incoming_vx, incoming_vy),
                         emit_fn,
-                    );
+                    )?;
                 }
 
                 // Render marker for the lone point; no directionality
@@ -715,7 +764,7 @@ where
                     MarkerType::Middle,
                     0.0,
                     emit_fn,
-                );
+                )?;
 
                 subpath_state = SubpathState::NoSubpath;
             }
@@ -732,7 +781,7 @@ where
                             MarkerType::Start,
                             angle_from_vector(outgoing_vx, outgoing_vy),
                             emit_fn,
-                        );
+                        )?;
 
                         subpath_state = SubpathState::InSubpath;
                     }
@@ -769,7 +818,7 @@ where
                             MarkerType::Middle,
                             angle,
                             emit_fn,
-                        );
+                        )?;
                     }
                 }
             }
@@ -802,9 +851,11 @@ where
                 MarkerType::End,
                 angle,
                 emit_fn,
-            );
+            )?;
         }
     }
+
+    Ok(())
 }
 
 // ************************************  Tests ************************************
@@ -1218,11 +1269,18 @@ mod marker_tests {
 
         let mut v = Vec::new();
 
-        emit_markers_for_path_builder(
-            &builder,
-            &mut |marker_type: MarkerType, x: f64, y: f64, computed_angle: f64| {
-                v.push((marker_type, x, y, computed_angle));
-            },
+        assert!(
+            emit_markers_for_path_builder(
+                &builder,
+                &mut |marker_type: MarkerType,
+                      x: f64,
+                      y: f64,
+                      computed_angle: f64|
+                 -> Result<(), RenderingError> {
+                    v.push((marker_type, x, y, computed_angle));
+                    Ok(())
+                }
+            ).is_ok()
         );
 
         assert_eq!(
@@ -1247,11 +1305,18 @@ mod marker_tests {
 
         let mut v = Vec::new();
 
-        emit_markers_for_path_builder(
-            &builder,
-            &mut |marker_type: MarkerType, x: f64, y: f64, computed_angle: f64| {
-                v.push((marker_type, x, y, computed_angle));
-            },
+        assert!(
+            emit_markers_for_path_builder(
+                &builder,
+                &mut |marker_type: MarkerType,
+                      x: f64,
+                      y: f64,
+                      computed_angle: f64|
+                 -> Result<(), RenderingError> {
+                    v.push((marker_type, x, y, computed_angle));
+                    Ok(())
+                }
+            ).is_ok()
         );
 
         assert_eq!(
