@@ -6,6 +6,7 @@ use std::cell::Cell;
 use attributes::Attribute;
 use coord_units::CoordUnits;
 use drawing_ctx::DrawingCtx;
+use error::RenderingError;
 use handle::RsvgHandle;
 use length::{Length, LengthDir};
 use node::{NodeResult, NodeTrait, RsvgNode};
@@ -62,17 +63,14 @@ impl NodeMask {
         node: &RsvgNode,
         affine_before_mask: &cairo::Matrix,
         draw_ctx: &mut DrawingCtx,
-    ) {
+    ) -> Result<(), RenderingError> {
         let cascaded = node.get_cascaded_values();
         let values = cascaded.get();
 
         let width = draw_ctx.get_width() as i32;
         let height = draw_ctx.get_height() as i32;
 
-        let mut surface = match cairo::ImageSurface::create(cairo::Format::ARgb32, width, height) {
-            Ok(surface) => surface,
-            Err(_) => return,
-        };
+        let mut surface = cairo::ImageSurface::create(cairo::Format::ARgb32, width, height)?;
 
         let mask_units = CoordUnits::from(self.units.get());
         let content_units = CoordUnits::from(self.content_units.get());
@@ -93,6 +91,16 @@ impl NodeMask {
         // Use a scope because mask_cr needs to release the
         // reference to the surface before we access the pixels
         {
+            let bbox_rect = {
+                if let Some(ref rect) = draw_ctx.get_bbox().rect {
+                    *rect
+                } else {
+                    // The node being masked is empty / doesn't have a
+                    // bounding box, so there's nothing to mask!
+                    return Ok(());
+                }
+            };
+
             let save_cr = draw_ctx.get_cairo_context();
 
             let mask_cr = cairo::Context::new(&surface);
@@ -102,36 +110,33 @@ impl NodeMask {
             draw_ctx.set_cairo_context(&mask_cr);
 
             if mask_units == CoordUnits::ObjectBoundingBox {
-                let rect = {
-                    let bbox = draw_ctx.get_bbox();
-                    bbox.rect.unwrap()
-                };
-
                 draw_ctx.clip(
-                    x * rect.width + rect.x,
-                    y * rect.height + rect.y,
-                    w * rect.width,
-                    h * rect.height,
+                    x * bbox_rect.width + bbox_rect.x,
+                    y * bbox_rect.height + bbox_rect.y,
+                    w * bbox_rect.width,
+                    h * bbox_rect.height,
                 );
             } else {
                 draw_ctx.clip(x, y, w, h);
             }
 
             if content_units == CoordUnits::ObjectBoundingBox {
-                let rect = {
-                    let bbox = draw_ctx.get_bbox();
-                    bbox.rect.unwrap()
-                };
-                let bbtransform =
-                    cairo::Matrix::new(rect.width, 0.0, 0.0, rect.height, rect.x, rect.y);
+                let bbtransform = cairo::Matrix::new(
+                    bbox_rect.width,
+                    0.0,
+                    0.0,
+                    bbox_rect.height,
+                    bbox_rect.x,
+                    bbox_rect.y,
+                );
 
                 mask_cr.transform(bbtransform);
 
                 draw_ctx.push_view_box(1.0, 1.0);
             }
 
-            draw_ctx.with_discrete_layer(node, values, false, &mut |dc| {
-                node.draw_children(&cascaded, dc, false);
+            let res = draw_ctx.with_discrete_layer(node, values, false, &mut |dc| {
+                node.draw_children(&cascaded, dc, false)
             });
 
             if content_units == CoordUnits::ObjectBoundingBox {
@@ -139,7 +144,9 @@ impl NodeMask {
             }
 
             draw_ctx.set_cairo_context(&save_cr);
-        }
+
+            res
+        }?;
 
         {
             let rowstride = surface.get_stride() as usize;
@@ -182,6 +189,8 @@ impl NodeMask {
 
         let (xofs, yofs) = draw_ctx.get_offset();
         cairo_mask_surface(&cr, &surface, xofs, yofs);
+
+        Ok(())
     }
 }
 
