@@ -8,9 +8,10 @@ use std::str::FromStr;
 
 use attributes::Attribute;
 use error::*;
+use font_props::{FontSizeSpec, FontWeightSpec, LetterSpacingSpec, SingleFontFamily};
 use handle::RsvgHandle;
 use iri::IRI;
-use length::{Dasharray, FontSizeSpec, Length, LengthDir, LengthUnit};
+use length::{Dasharray, Length, LengthDir, LengthUnit};
 use node::RsvgNode;
 use paint_server::PaintServer;
 use parsers::{Parse, ParseError};
@@ -44,7 +45,7 @@ where
     T: Property<ComputedValues> + Clone + Default,
 {
     pub fn compute(&self, src: &T, src_values: &ComputedValues) -> T {
-        match *self {
+        let value: T = match *self {
             SpecifiedValue::Unspecified => {
                 if <T as Property<ComputedValues>>::inherits_automatically() {
                     src.clone()
@@ -55,8 +56,10 @@ where
 
             SpecifiedValue::Inherit => src.clone(),
 
-            SpecifiedValue::Specified(ref v) => v.compute(src_values),
-        }
+            SpecifiedValue::Specified(ref v) => v.clone(),
+        };
+
+        value.compute(src_values)
     }
 }
 
@@ -454,7 +457,7 @@ impl State {
                 }
 
                 Attribute::LetterSpacing => {
-                    self.values.letter_spacing = parse_property(value, LengthDir::Horizontal)?;
+                    self.values.letter_spacing = parse_property(value, ())?;
                 }
 
                 Attribute::LightingColor => {
@@ -587,10 +590,19 @@ impl State {
         // https://www.w3.org/TR/CSS2/syndata.html#unsupported-values
         // Ignore unsupported / illegal values; don't set the whole
         // node to be in error in that case.
+
+        if let Err(e) = parse().map_err(|e| NodeError::attribute_error(attr, e)) {
+            rsvg_log!(
+                "(style property error for attribute {:?}\n    value=\"{}\"\n    {}\n    property \
+                 will be ignored)",
+                attr,
+                value,
+                e
+            );
+        }
+
+        // If we didn't ignore property errors, we could just return this:
         // parse().map_err(|e| NodeError::attribute_error(attr, e))
-
-        let _ = parse();
-
         Ok(())
     }
 
@@ -618,11 +630,6 @@ impl State {
                 let value = value[1..].trim();
 
                 if !prop_name.is_empty() && !value.is_empty() {
-                    // Just remove single quotes in a trivial way.  No handling for any
-                    // special character inside the quotes is done.  This relates
-                    // especially to font-family names.
-                    let value = value.replace('\'', "");
-
                     let mut important = false;
 
                     let value = if let Some(bang_pos) = value.find('!') {
@@ -942,9 +949,9 @@ make_property!(
 make_property!(
     ComputedValues,
     FontFamily,
-    default: "Times New Roman".to_string(),
+    default: SingleFontFamily("Times New Roman".to_string()),
     inherits_automatically: true,
-    newtype_parse: String,
+    newtype_parse: SingleFontFamily,
     parse_data_type: ()
 );
 
@@ -1014,37 +1021,34 @@ make_property!(
     "small-caps" => SmallCaps,
 );
 
-// https://www.w3.org/TR/SVG/text.html#FontWeightProperty
+// https://www.w3.org/TR/2008/REC-CSS2-20080411/fonts.html#propdef-font-weight
 make_property!(
     ComputedValues,
     FontWeight,
-    default: Normal,
+    default: FontWeightSpec::Normal,
     inherits_automatically: true,
-
-    identifiers:
-    "normal" => Normal,
-    "bold" => Bold,
-    "bolder" => Bolder,
-    "lighter" => Lighter,
-    "100" => W100, // FIXME: we should use Weight(100),
-    "200" => W200, // but we need a smarter macro for that
-    "300" => W300,
-    "400" => W400,
-    "500" => W500,
-    "600" => W600,
-    "700" => W700,
-    "800" => W800,
-    "900" => W900,
+    newtype_parse: FontWeightSpec,
+    parse_data_type: ()
 );
 
 // https://www.w3.org/TR/SVG/text.html#LetterSpacingProperty
 make_property!(
     ComputedValues,
     LetterSpacing,
-    default: Length::default(),
-    inherits_automatically: true,
-    newtype_parse: Length,
-    parse_data_type: LengthDir
+    default: LetterSpacingSpec::Normal,
+    newtype_parse: LetterSpacingSpec,
+    parse_data_type: (),
+    property_impl: {
+        impl Property<ComputedValues> for LetterSpacing {
+            fn inherits_automatically() -> bool {
+                true
+            }
+
+            fn compute(&self, _v: &ComputedValues) -> Self {
+                LetterSpacing(self.0.compute())
+            }
+        }
+    }
 );
 
 // https://www.w3.org/TR/SVG/filters.html#LightingColorProperty
@@ -1508,17 +1512,19 @@ pub fn parse_style_attrs(
 
     match state.parse_presentation_attributes(pbag) {
         Ok(_) => (),
-        Err(_) => (),
-        /* FIXME: we'll ignore errors here for now.  If we return, we expose
-         * buggy handling of the enable-background property; we are not parsing it correctly.
-         * This causes tests/fixtures/reftests/bugs/587721-text-transform.svg to fail
-         * because it has enable-background="new 0 0 1179.75118 687.74173" in the toplevel svg
-         * element.
-         *        Err(e) => (),
-         *        {
-         *            node.set_error(e);
-         *            return;
-         *        } */
+        Err(e) => {
+            // FIXME: we'll ignore errors here for now.  If we return, we expose
+            // buggy handling of the enable-background property; we are not parsing it correctly.
+            // This causes tests/fixtures/reftests/bugs/587721-text-transform.svg to fail
+            // because it has enable-background="new 0 0 1179.75118 687.74173" in the toplevel svg
+            // element.
+            //        {
+            //            node.set_error(e);
+            //            return;
+            //        }
+
+            rsvg_log!("(attribute error: {})", e);
+        }
     }
 
     // Try to properly support all of the following, including inheritance:
