@@ -1,4 +1,5 @@
 //! Shared access to Cairo image surfaces.
+use std::cmp::min;
 use std::marker::PhantomData;
 use std::ptr::NonNull;
 
@@ -70,6 +71,42 @@ pub struct SharedImageSurface {
 // The access is read-only, the ref-counting on an `ImageSurface` is atomic.
 unsafe impl Sync for SharedImageSurface {}
 
+/// A compile-time blur direction variable.
+pub trait BlurDirection {
+    const IS_VERTICAL: bool;
+}
+
+/// Vertical blur direction.
+pub enum Vertical {}
+/// Horizontal blur direction.
+pub enum Horizontal {}
+
+impl BlurDirection for Vertical {
+    const IS_VERTICAL: bool = true;
+}
+
+impl BlurDirection for Horizontal {
+    const IS_VERTICAL: bool = false;
+}
+
+/// A compile-time alpha-only marker variable.
+pub trait IsAlphaOnly {
+    const IS_ALPHA_ONLY: bool;
+}
+
+/// Alpha-only.
+pub enum AlphaOnly {}
+/// Not alpha-only.
+pub enum NotAlphaOnly {}
+
+impl IsAlphaOnly for AlphaOnly {
+    const IS_ALPHA_ONLY: bool = true;
+}
+
+impl IsAlphaOnly for NotAlphaOnly {
+    const IS_ALPHA_ONLY: bool = false;
+}
+
 impl SharedImageSurface {
     /// Creates a `SharedImageSurface` from a unique `ImageSurface`.
     ///
@@ -141,6 +178,12 @@ impl SharedImageSurface {
         self.height
     }
 
+    /// Returns the surface stride.
+    #[inline]
+    pub fn stride(&self) -> isize {
+        self.stride
+    }
+
     /// Returns `true` if the surface contains meaningful data only in the alpha channel.
     #[inline]
     pub fn is_alpha_only(&self) -> bool {
@@ -169,20 +212,13 @@ impl SharedImageSurface {
         Pixel::from_u32(value)
     }
 
-    /// Retrieves the pixel value if it is within `bounds`, otherwise returns a transparent black
-    /// pixel.
+    /// Retrieves the pixel value by offset into the pixel data array.
     #[inline]
-    pub fn get_pixel_or_transparent(&self, bounds: IRect, x: i32, y: i32) -> Pixel {
-        if bounds.contains(x, y) {
-            self.get_pixel(x as u32, y as u32)
-        } else {
-            Pixel {
-                r: 0,
-                g: 0,
-                b: 0,
-                a: 0,
-            }
-        }
+    pub fn get_pixel_by_offset(&self, offset: isize) -> Pixel {
+        assert!(offset < self.stride as isize * self.height as isize);
+
+        let value = unsafe { *(self.data_ptr.as_ptr().offset(offset) as *const u32) };
+        Pixel::from_u32(value)
     }
 
     /// Calls `set_source_surface()` on the given Cairo context.
@@ -376,7 +412,7 @@ impl SharedImageSurface {
                         a += f64::from(pixel.a) * factor;
                     }
 
-                    let convert = |x: f64| clamp(x, 0.0, 255.0).round() as u8;
+                    let convert = |x: f64| (clamp(x, 0.0, 255.0) + 0.5) as u8;
 
                     let output_pixel = Pixel {
                         r: 0,
@@ -413,7 +449,7 @@ impl SharedImageSurface {
                         a += f64::from(pixel.a) * factor;
                     }
 
-                    let convert = |x: f64| clamp(x, 0.0, 255.0).round() as u8;
+                    let convert = |x: f64| (clamp(x, 0.0, 255.0) + 0.5) as u8;
 
                     let output_pixel = Pixel {
                         r: convert(r),
@@ -441,16 +477,16 @@ impl SharedImageSurface {
     /// Panics if `kernel_size` is `0` or if `target >= kernel_size`.
     // This is public (and not inlined into box_blur()) for the purpose of accessing it from the
     // benchmarks.
-    pub fn box_blur_loop(
+    pub fn box_blur_loop<B: BlurDirection, A: IsAlphaOnly>(
         &self,
         output_surface: &mut cairo::ImageSurface,
         bounds: IRect,
         kernel_size: usize,
         target: usize,
-        vertical: bool,
     ) {
         assert_ne!(kernel_size, 0);
         assert!(target < kernel_size);
+        assert_eq!(self.is_alpha_only(), A::IS_ALPHA_ONLY);
 
         {
             // The following code is needed for a parallel implementation of the blur loop. The
@@ -481,8 +517,8 @@ impl SharedImageSurface {
                 /// Creates a new `UnsafeSendPixelData`.
                 ///
                 /// # Safety
-                /// You must call cairo_surface_mark_dirty() on the surface once all instances of
-                /// `UnsafeSendPixelData` are dropped to make sure the pixel changes are comitted
+                /// You must call `cairo_surface_mark_dirty()` on the surface once all instances of
+                /// `UnsafeSendPixelData` are dropped to make sure the pixel changes are committed
                 /// to Cairo.
                 #[inline]
                 unsafe fn new(surface: &mut cairo::ImageSurface) -> Self {
@@ -518,7 +554,7 @@ impl SharedImageSurface {
 
                 /// Splits this `UnsafeSendPixelData` into two at the given row.
                 ///
-                /// The first one contains rows `0..index` (index not included) and the second one
+                /// The first one contains rows `0..index` (`index` not included) and the second one
                 /// contains rows `index..height`.
                 #[inline]
                 fn split_at_row(self, index: u32) -> (Self, Self) {
@@ -546,7 +582,7 @@ impl SharedImageSurface {
 
                 /// Splits this `UnsafeSendPixelData` into two at the given column.
                 ///
-                /// The first one contains columns `0..index` (index not included) and the second
+                /// The first one contains columns `0..index` (`index` not included) and the second
                 /// one contains columns `index..width`.
                 #[inline]
                 fn split_at_column(self, index: u32) -> (Self, Self) {
@@ -588,7 +624,7 @@ impl SharedImageSurface {
             // which the blurring happens (so if `vertical` is false, the main axis is the
             // horizontal axis). The other axis is the outer loop axis. The code uses `i` and `j`
             // for the other axis and main axis coordinates, respectively.
-            let (main_axis_min, main_axis_max, other_axis_min, other_axis_max) = if vertical {
+            let (main_axis_min, main_axis_max, other_axis_min, other_axis_max) = if B::IS_VERTICAL {
                 (bounds.y0, bounds.y1, bounds.x0, bounds.x1)
             } else {
                 (bounds.x0, bounds.x1, bounds.y0, bounds.y1)
@@ -596,14 +632,14 @@ impl SharedImageSurface {
 
             // Helper function for getting the pixels.
             let pixel = |i, j| {
-                let (x, y) = if vertical { (i, j) } else { (j, i) };
+                let (x, y) = if B::IS_VERTICAL { (i, j) } else { (j, i) };
 
-                self.get_pixel_or_transparent(bounds, x, y)
+                self.get_pixel(x as u32, y as u32)
             };
 
             // The following loop assumes the first row or column of `output_data` is the first row
             // or column inside `bounds`.
-            let mut output_data = if vertical {
+            let mut output_data = if B::IS_VERTICAL {
                 output_data.split_at_column(bounds.x0 as u32).1
             } else {
                 output_data.split_at_row(bounds.y0 as u32).1
@@ -614,7 +650,7 @@ impl SharedImageSurface {
                     // Split off one row or column and launch its processing on another thread.
                     // Thanks to the initial split before the loop, there's no special case for the
                     // very first split.
-                    let (mut current, remaining) = if vertical {
+                    let (mut current, remaining) = if B::IS_VERTICAL {
                         output_data.split_at_column(1)
                     } else {
                         output_data.split_at_row(1)
@@ -627,7 +663,7 @@ impl SharedImageSurface {
                         let mut set_pixel = |j, pixel| {
                             // We're processing rows or columns one-by-one, so the other coordinate
                             // is always 0.
-                            let (x, y) = if vertical { (0, j) } else { (j, 0) };
+                            let (x, y) = if B::IS_VERTICAL { (0, j) } else { (j, 0) };
                             current.set_pixel(pixel, x, y);
                         };
 
@@ -645,10 +681,10 @@ impl SharedImageSurface {
                         // The whole sum needs to be computed for the first pixel. However, we know
                         // that values outside of bounds are transparent, so the loop starts on the
                         // first pixel in bounds.
-                        for j in main_axis_min..main_axis_min + shift {
+                        for j in main_axis_min..min(main_axis_max, main_axis_min + shift) {
                             let Pixel { r, g, b, a } = pixel(i, j);
 
-                            if !self.is_alpha_only() {
+                            if !A::IS_ALPHA_ONLY {
                                 sum_r += u32::from(r);
                                 sum_g += u32::from(g);
                                 sum_b += u32::from(b);
@@ -668,27 +704,40 @@ impl SharedImageSurface {
                         );
 
                         // Now, go through all the other pixels.
+                        //
+                        // j - target - 1 >= main_axis_min
+                        // j >= main_axis_min + target + 1
+                        let start_subtracting_at = main_axis_min + target + 1;
+
+                        // j + shift - 1 < main_axis_max
+                        // j < main_axis_max - shift + 1
+                        let stop_adding_at = main_axis_max - shift + 1;
+
                         for j in main_axis_min + 1..main_axis_max {
-                            let old_pixel = pixel(i, j - target - 1);
+                            if j >= start_subtracting_at {
+                                let old_pixel = pixel(i, j - target - 1);
 
-                            if !self.is_alpha_only() {
-                                sum_r -= u32::from(old_pixel.r);
-                                sum_g -= u32::from(old_pixel.g);
-                                sum_b -= u32::from(old_pixel.b);
+                                if !A::IS_ALPHA_ONLY {
+                                    sum_r -= u32::from(old_pixel.r);
+                                    sum_g -= u32::from(old_pixel.g);
+                                    sum_b -= u32::from(old_pixel.b);
+                                }
+
+                                sum_a -= u32::from(old_pixel.a);
                             }
 
-                            sum_a -= u32::from(old_pixel.a);
+                            if j < stop_adding_at {
+                                let new_pixel = pixel(i, j + shift - 1);
 
-                            let new_pixel = pixel(i, j + shift - 1);
+                                if !A::IS_ALPHA_ONLY {
+                                    sum_r += u32::from(new_pixel.r);
+                                    sum_g += u32::from(new_pixel.g);
+                                    sum_b += u32::from(new_pixel.b);
+                                }
 
-                            if !self.is_alpha_only() {
-                                sum_r += u32::from(new_pixel.r);
-                                sum_g += u32::from(new_pixel.g);
-                                sum_b += u32::from(new_pixel.b);
+                                sum_a += u32::from(new_pixel.a);
                             }
 
-                            sum_a += u32::from(new_pixel.a);
-
                             set_pixel(
                                 j as u32,
                                 Pixel {
@@ -719,17 +768,20 @@ impl SharedImageSurface {
     /// # Panics
     /// Panics if `kernel_size` is `0` or if `target >= kernel_size`.
     #[inline]
-    pub fn box_blur(
+    pub fn box_blur<B: BlurDirection>(
         &self,
         bounds: IRect,
         kernel_size: usize,
         target: usize,
-        vertical: bool,
     ) -> Result<SharedImageSurface, cairo::Status> {
         let mut output_surface =
             cairo::ImageSurface::create(cairo::Format::ARgb32, self.width, self.height)?;
 
-        self.box_blur_loop(&mut output_surface, bounds, kernel_size, target, vertical);
+        if self.is_alpha_only() {
+            self.box_blur_loop::<B, AlphaOnly>(&mut output_surface, bounds, kernel_size, target);
+        } else {
+            self.box_blur_loop::<B, NotAlphaOnly>(&mut output_surface, bounds, kernel_size, target);
+        }
 
         SharedImageSurface::new(output_surface, self.surface_type)
     }
