@@ -98,8 +98,7 @@ fn parse_matrix_args(parser: &mut Parser) -> Result<cairo::Matrix, AttributeErro
             let y0 = f64::from(p.expect_number()?);
 
             Ok(cairo::Matrix::new(xx, yx, xy, yy, x0, y0))
-        })
-        .map_err(CssParseError::<()>::basic)
+        }).map_err(CssParseError::<()>::basic)
         .map_err(AttributeError::from)
 }
 
@@ -116,8 +115,7 @@ fn parse_translate_args(parser: &mut Parser) -> Result<cairo::Matrix, AttributeE
             );
 
             Ok(cairo::Matrix::new(1.0, 0.0, 0.0, 1.0, tx, ty))
-        })
-        .map_err(CssParseError::<()>::basic)
+        }).map_err(CssParseError::<()>::basic)
         .map_err(AttributeError::from)
 }
 
@@ -126,16 +124,15 @@ fn parse_scale_args(parser: &mut Parser) -> Result<cairo::Matrix, AttributeError
         .parse_nested_block(|p| {
             let x = f64::from(p.expect_number()?);
 
-            let y =
-                p.try(|p| -> Result<f32, CssParseError<()>> {
+            let y = p
+                .try(|p| -> Result<f32, CssParseError<()>> {
                     optional_comma(p);
                     Ok(p.expect_number()?)
                 }).map(f64::from)
-                    .unwrap_or(x);
+                .unwrap_or(x);
 
             Ok(cairo::Matrix::new(x, 0.0, 0.0, y, 0.0, 0.0))
-        })
-        .map_err(CssParseError::<()>::basic)
+        }).map_err(CssParseError::<()>::basic)
         .map_err(AttributeError::from)
 }
 
@@ -145,8 +142,8 @@ fn parse_rotate_args(parser: &mut Parser) -> Result<cairo::Matrix, AttributeErro
             let angle = f64::from(p.expect_number()?) * PI / 180.0;
             let (s, c) = angle.sin_cos();
 
-            let (tx, ty) =
-                p.try(|p| -> Result<_, CssParseError<()>> {
+            let (tx, ty) = p
+                .try(|p| -> Result<_, CssParseError<()>> {
                     optional_comma(p);
                     let tx = f64::from(p.expect_number()?);
 
@@ -161,8 +158,7 @@ fn parse_rotate_args(parser: &mut Parser) -> Result<cairo::Matrix, AttributeErro
             m = cairo::Matrix::multiply(&cairo::Matrix::new(c, s, -s, c, 0.0, 0.0), &m);
             m = cairo::Matrix::multiply(&cairo::Matrix::new(1.0, 0.0, 0.0, 1.0, -tx, -ty), &m);
             Ok(m)
-        })
-        .map_err(CssParseError::<()>::basic)
+        }).map_err(CssParseError::<()>::basic)
         .map_err(AttributeError::from)
 }
 
@@ -171,8 +167,7 @@ fn parse_skewx_args(parser: &mut Parser) -> Result<cairo::Matrix, AttributeError
         .parse_nested_block(|p| {
             let a = f64::from(p.expect_number()?) * PI / 180.0;
             Ok(cairo::Matrix::new(1.0, 0.0, a.tan(), 1.0, 0.0, 0.0))
-        })
-        .map_err(CssParseError::<()>::basic)
+        }).map_err(CssParseError::<()>::basic)
         .map_err(AttributeError::from)
 }
 
@@ -181,8 +176,7 @@ fn parse_skewy_args(parser: &mut Parser) -> Result<cairo::Matrix, AttributeError
         .parse_nested_block(|p| {
             let a = f64::from(p.expect_number()?) * PI / 180.0;
             Ok(cairo::Matrix::new(1.0, a.tan(), 0.0, 1.0, 0.0, 0.0))
-        })
-        .map_err(CssParseError::<()>::basic)
+        }).map_err(CssParseError::<()>::basic)
         .map_err(AttributeError::from)
 }
 
