@@ -8,8 +8,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <glib.h>
-#include <rsvg.h>
-#include "rsvg-compat.h"
+#include "librsvg/rsvg.h"
 
 static void
 show_help (GOptionContext *context)
@@ -41,8 +40,6 @@ main (int	  argc,
         { NULL }
     };
 
-    RSVG_G_TYPE_INIT;
-
     context = NULL;
     fragment = NULL;
     filenames = NULL;
